<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Auto Inspection</title>
</head>
<body>
    <a href="/"> home</a>
    <a href="/change_model">change_model</a>
    <h1>Auto Inspection API</h1>
    <form method="POST">
        <button type="submit" name="button" value="Capture">Capture</button>
        <button type="submit" name="button" value="Auto">Auto</button>
        <button type="submit" name="button" value="Adj">Adj</button>
        <button type="submit" name="button" value="Predict">Predict</button>
        <button type="submit" name="button" value="Capture&Predict">Capture&Predict</button>
    </form>
</body>
</html>